import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Logger;

import org.hornetq.api.core.HornetQException;
import org.hornetq.api.core.SimpleString;
import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.core.client.ClientMessage;
import org.hornetq.api.core.client.ClientProducer;
import org.hornetq.api.core.client.ClientSession;
import org.hornetq.api.core.client.ClientSessionFactory;
import org.hornetq.api.core.client.HornetQClient;
import org.hornetq.api.core.management.QueueControl;
import org.hornetq.api.core.management.ResourceNames;
import org.hornetq.core.config.Configuration;
import org.hornetq.core.config.impl.ConfigurationImpl;
import org.hornetq.core.remoting.impl.invm.InVMAcceptorFactory;
import org.hornetq.core.remoting.impl.invm.InVMConnectorFactory;
import org.hornetq.core.server.HornetQServer;
import org.hornetq.core.server.HornetQServers;
import org.hornetq.core.server.JournalType;
import org.hornetq.core.settings.impl.AddressFullMessagePolicy;
import org.hornetq.core.settings.impl.AddressSettings;

public class Test {

	static final Logger LOG = Logger.getLogger(Test.class.getName());

	public static final String QUEUE_NAME = "queue.xyz";
	public static final String DLQ_QUEUE_NAME = "queue.DLQ";
	private QueueControl queueControl;
	private QueueControl dlqControl;
	private ExecutorService executor = Executors.newScheduledThreadPool(5);
	ClientSession coreSession = null;
	HornetQServer server = null;


	public Test() {
		try {
			Configuration configuration = new ConfigurationImpl();
			configuration.setPersistenceEnabled(true);
			configuration.setSecurityEnabled(false);
			configuration.setBindingsDirectory("D:/hq/bindings");
			configuration.setJournalDirectory("D:/hq/journal");
			configuration.setPagingDirectory("D:/hq/paging");
			configuration.setJournalType(JournalType.NIO);

			//configuration.set
			configuration.getAcceptorConfigurations()
				.add(
					new TransportConfiguration(InVMAcceptorFactory.class
						.getName()));

			server = HornetQServers.newHornetQServer(configuration);

			AddressSettings settings = new AddressSettings();
			settings.setDeadLetterAddress(new SimpleString(DLQ_QUEUE_NAME));
			settings.setMaxSizeBytes(1024 * 1024);
			settings.setPageSizeBytes(100 * 1024);
			settings.setAddressFullMessagePolicy(AddressFullMessagePolicy.PAGE);
			//settings.setExpiryAddress(new SimpleString("jms.queue.DLQ"));
			//settings.setRedeliveryDelay(5000);
			settings.setMaxDeliveryAttempts(3);
			server.getAddressSettingsRepository().setDefault(settings);
			server.getAddressSettingsRepository().addMatch("#", settings);

			server.start();

			// Step 3. As we are not using a JNDI environment we instantiate the objects directly
			ClientSessionFactory sf = HornetQClient
				.createClientSessionFactory(new TransportConfiguration(
					InVMConnectorFactory.class.getName()));

			// Step 4. Create a core queue

			coreSession = sf.createSession(false, false, false);

			//final String queueName = "queue.TestQueue4";
			//coreSession.queueQuery()

			try {
				coreSession.createQueue(QUEUE_NAME, QUEUE_NAME, true);
			} catch (HornetQException e) {
				if (e.getCode() == 101) {
					LOG.info("Queue [" + QUEUE_NAME + "] already exists.");
				} else {
					e.printStackTrace();
				}
			}

			try {
				coreSession.createQueue(DLQ_QUEUE_NAME, DLQ_QUEUE_NAME, true);
			} catch (HornetQException e) {
				if (e.getCode() == 101) {
					LOG.info("Queue [" + DLQ_QUEUE_NAME + "] already exists.");
				} else {
					e.printStackTrace();
				}
			}

			//			// Step 7. Use JMX to retrieve the message counters using the JMSQueueControl

			//			ObjectName on = ObjectNameBuilder.DEFAULT
			//				.getJMSQueueObjectName(ResourceNames.JMS_QUEUE + QUEUE_NAME);
			//			JMSQueueControl jmsQueueControl = (JMSQueueControl) MBeanServerInvocationHandler
			//				.newProxyInstance(server.getMBeanServer(), on,
			//					JMSQueueControl.class, false);
			//			LOG.info("jmsQueueControl MessageCount: "
			//				+ jmsQueueControl.getMessageCount());

			queueControl = (QueueControl) server.getManagementService()
				.getResource(ResourceNames.CORE_QUEUE + QUEUE_NAME);
			dlqControl = (QueueControl) server.getManagementService()
				.getResource(ResourceNames.CORE_QUEUE + DLQ_QUEUE_NAME);

			LOG.info("MessageCount: " + queueControl.getMessageCount());
			LOG.info("DeadLetterAddress: "
				+ queueControl.getDeadLetterAddress());

			queueControl.setDeadLetterAddress(DLQ_QUEUE_NAME);
			LOG.info("DeadLetterAddress: "
				+ queueControl.getDeadLetterAddress());

			LOG.info("**********Server Started***********");
			LOG.info(" initialised with JMS Queue");
		} catch (Exception ex) {
			ex.printStackTrace();
			stop();
		}
	}

	public void stop() {
		if (coreSession != null) {
			try {
				coreSession.close();
			} catch (HornetQException e) {
				e.printStackTrace();
			}
		}
		if (server != null) {
			try {
				server.stop();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void test() throws InterruptedException, BrokenBarrierException {
		int producersCount = 1;
		CyclicBarrier cb = new CyclicBarrier(producersCount + 1);

		for (int i = 0; i < producersCount; i++) {
			executor.execute((new Producer(QUEUE_NAME, 10, cb)));
		}
		LOG.info("wait...");
		cb.await();
		LOG.info("wait..." + "OK");
		executor.shutdown();
	}


	public class Producer implements Runnable {

		private int count;
		private final String queueName;
		private final CyclicBarrier cb;


		public Producer(String queueName, int count, CyclicBarrier cb) {
			this.queueName = queueName;
			this.count = count;
			this.cb = cb;
		}

		@Override
		public void run() {
			ClientSessionFactory sf = HornetQClient
				.createClientSessionFactory(new TransportConfiguration(
					InVMConnectorFactory.class.getName()));

			ClientSession session = null;

			try {
				// Step 5. Create the session, and producer
				session = sf.createSession();
				ClientProducer producer = session.createProducer(queueName);
				// Step 6. Create and send a message
				long time = System.currentTimeMillis();
				for (int msgIndex = 0; msgIndex < count; msgIndex++) {
					String text = "Message#" + msgIndex;

					ClientMessage message = session.createMessage(true);
					final String propName = "myprop";
					message.putStringProperty(propName, text);
					//				message.putObjectProperty(arg0, arg1)
					System.out.println(Thread.currentThread().getName()
						+ " send:" + text);
					producer.send(message);
				}
				time = System.currentTimeMillis() - time;
				System.out.println(Thread.currentThread().getName()
					+ " time to send [" + count + "] messages : " + time
					+ " ms" + "\r\n" + (count * 1000 / time)
					+ " messages per second");
			} catch (Exception ex) {
				ex.printStackTrace();
			} finally {
				if (session != null) {
					try {
						session.close();
					} catch (HornetQException e) {
						e.printStackTrace(); //To change body of catch statement use File | Settings | File Templates.
					}
				}
				try {
					cb.await();
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}

		}

	}


	public static void main(String[] args) throws InterruptedException,
		BrokenBarrierException {
		Test tc = new Test();
		try {
			tc.test();
		} finally {
			tc.stop();
		}
	}
}
